/*
 * This is source code of the Snapshot Generator, an extension for USE
 * to generate (valid) system states of UML models.
 * Copyright (C) 2001 Joern Bohling, University of Bremen
 *
 * About USE:
 *   USE - UML based specification environment
 *   Copyright (C) 1999,2000,2001 Mark Richters, University of Bremen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * March 22th 2001 
 * @author  Joern Bohling
 */

package org.tzi.use.parser.generator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import antlr.CommonToken;
import org.tzi.use.gen.assl.statics.GInstruction;
import org.tzi.use.gen.assl.statics.GInstructionCreator;
import org.tzi.use.gen.assl.statics.GValueInstruction;
import org.tzi.use.parser.Context;
import org.tzi.use.parser.MyToken;
import org.tzi.use.parser.SemanticException;
import org.tzi.use.util.StringUtil;

public class ASTGAtomicInstruction extends ASTGValueInstruction {
    MyToken fName;
    List fParameter;  // ASTGInstructionParameterInterface;

    public ASTGAtomicInstruction(MyToken name) {
        fName = name;
        fParameter = new ArrayList();
    }

    public void addParameter( ASTGInstructionParameterInterface parameter ) {
        fParameter.add(parameter);
    }

    public GInstruction gen(Context ctx) throws SemanticException {

        List params = new ArrayList();
        List errParams = new ArrayList();

        Iterator it = fParameter.iterator();
        while (it.hasNext() ) {
            Object param = it.next();
            if (param instanceof CommonToken ) {
                String name = ((CommonToken) param).getText();
                boolean isClass = ctx.model().getClass(name)!=null;
                boolean isAssociation = ctx.model().getAssociation(name)!=null;
                if (isClass && isAssociation )
                    errParams.add("Class/Association");
                else if (isClass)
                    errParams.add("Class");
                else if (isAssociation)
                    errParams.add("Association");
                else {
                    String err = "`" + name +"' is not a class or " +
                        "association of the current model. If `" + name +
                        "' is a variable, use squared " +
                        "brackets.";
                    throw new SemanticException(fName, err);
                }
                params.add( name );
            } else {
                GValueInstruction instr =
                    (GValueInstruction) ((ASTGocl) param).gen(ctx);
                params.add( instr );
                errParams.add( instr.type().toString() );
            }
        }

        // params is a list over Strings or GValueInstructions.
        // A containing string is a classname or associationname.

        GInstruction instr;
        instr = GInstructionCreator.instance().create( fName.getText(), 
                                                       params, 
                                                       ctx.model() );
        if (instr == null) {
            String err = "Instruction `"+ fName.getText() +"(" + 
                StringUtil.fmtSeq(errParams.iterator(), ",") +
                ")' not found.";
            throw new SemanticException(fName, err);
        }
        return (GInstruction) instr;
    }
}
